<?php
/**
 * @file
 * @param $form, $form_state
 * @return $form
 */
function _jwplayer_report_settings_form() {
  $form = array();

  $form['property_name_key'] = array(
    '#type' => 'textfield',
    '#required' => TRUE,
    '#default_value' => variable_get('property_name_key', ''),
    '#title' => t('Property Name Key'),
    '#description' => t('Enter Property Name Key.'),
  );

  $form['api_secret_key'] = array(
    '#type' => 'textfield',
    '#required' => TRUE,
    '#default_value' => variable_get('api_secret_key', ''),
    '#title' => t('API Secret Key'),
    '#description' => t('Enter API Secret Key.'),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Save'
  );
  return $form;
}


/**
 * @param $form, $form_state
 * @return none
 */
function _jwplayer_report_settings_form_submit($form, $form_state) {
  $api_secret_key = trim($form_state['values']['api_secret_key']);
  $property_name_key = trim($form_state['values']['property_name_key']);

  variable_set('api_secret_key', $api_secret_key);
  variable_set('property_name_key', $property_name_key);

  drupal_set_message(t('Settings saved succesfully'));
}

/**
 *  Implementation of hook_form() to create form elements.
 *
 * @return
 *   Constructed form.
 *
 * @ingroup forms
 * @see _jwplayer_report_form_validate()
 * @see _jwplayer_report_form_submit()
 */
function _jwplayer_report_form() {

  $module_path = drupal_get_path('module', 'jwplayer_report');
  drupal_add_js($module_path . '/js/jwplayer_report.js');
  drupal_add_css($module_path . '/css/jwplayer_report.css');
  $start_date_obj = new DateTime();
  $start_date = $start_date_obj->format('d-M-Y');
  $media_id = '';
  $get_report_range = '';

  if (isset($_POST['report_range'])) {
     $report_range = $_POST['report_range'];
   }
   else {
     $report_range = '';
   }

  $new_media_id = str_replace(',', '-', $media_id);

  $form['media_id'] = array(
  '#id' => 'media_id',
  '#title' => 'MediaId(s)',
  '#type' => 'textfield',
  '#description' => "Please enter single/comma-separated MediaId(s). If MediaId(s) is/are not available then the report will contain data for all MediaId's.",
  '#default_value' => isset($_GET['media_id']) ? str_replace('-', ',', $_GET['media_id']) : '',
  );

  if ((isset($_GET['month_val'])) && ($_GET['month_val'] == 6)) {
    $get_report_range = 1;
  }
  elseif ((isset($_GET['month_val'])) && ($_GET['month_val'] == 12)) {
    $get_report_range = 2;
  }
  elseif ((isset($_GET['month_val'])) && ($_GET['month_val'] == 24)) {
    $get_report_range = 3;
  }
  elseif ((isset($_GET['month_val'])) && ($_GET['month_val'] == 'custom')) {
    $get_report_range = 4;
  }

  $form['report_range'] = array(
  '#title' => t('Range'),
  '#type' => 'select',
  '#default_value' => isset($get_report_range) ? $get_report_range : $report_range,
  '#description' => 'Select report range',
  '#options' => array(
      0 => '-- Select --',
      1 => 'Past six months',
      2 => 'Past twelve months',
      3 => 'Past twenty four months',
      4 => 'Custom range',
   ),
  );

  if ($get_report_range == 4 || $report_range == 4) {
    $form['report_date'] = array(
    '#type' => 'fieldset',
    '#title' => t('Date selector'),
    '#prefix' => '<div id="show-hide-template">',
    '#suffix' => '</div>',
    '#collapsible' => FALSE,
    );
  }
  else {
    $form['report_date'] = array(
      '#type' => 'fieldset',
      '#title' => t('Date selector'),
      '#prefix' => '<div id="show-hide-template" style="display:none;">',
      '#suffix' => '</div>',
      '#collapsible' => FALSE,
    );

  }

  $def_pub_start_date = isset($_GET['start_date']) ? $_GET['start_date'] : date("Y-m-d", time());
  $def_pub_end_date = isset($_GET['end_date']) ? $_GET['end_date'] : date("Y-m-d", time());

  $form['report_date']['start_date'] = array(
  '#title' => t('Start Date'),
  '#type' => 'date_popup',
  '#date_label_position' => 'hidden',
  '#date_format' => 'Y-m-d',
  '#default_value' => $def_pub_start_date,
  '#prefix' => '<div class="fullwidth"><div class="halfwidth">',
  '#suffix' => '</div>',
  );

  $form['report_date']['end_date'] = array(
  '#title' => t('End Date'),
  '#type' => 'date_popup',
  '#date_label_position' => 'hidden',
  '#date_format' => 'Y-m-d',
  '#default_value' => $def_pub_end_date,
  '#prefix' => '<div class="fullwidth"><div class="halfwidth">',
  '#suffix' => '</div>',
  );

  $form['save'] = array(
  '#type' => 'submit',
  '#default_value' => t('Download'),
  );

  $form['list'] = array(
  '#type' => 'submit',
  '#default_value' => t('List'),
  );

  $form['reset'] = array(
  '#type' => 'submit',
  '#default_value' => t('Reset'),
  );

  $template_list = list_jwplayerreport();

  $form['data_table'] = array(
  '#name' => 'data_table',
  '#markup' => $template_list,
  );

  return $form;
}

/**
 *  Implementation of the hook_validate() to validate form fields
 * @param $form
 * @param $form_state
 */
function _jwplayer_report_form_validate($form, &$form_state) {
  $start_date_form = strtotime($form_state['values']['start_date']);
  $end_date_form = strtotime($form_state['values']['end_date']);
  $media_id = $form_state['values']['media_id'];
  $report_range = $form_state['values']['report_range'];
  $option = $form_state['values']['op'];

  if ($option == 'Reset') {
    drupal_goto('admin/jwplayer/report');
  }

  if (strpos($media_id, ' ') !== FALSE) {
    form_set_error('media_id', 'Please enter mediaId(s) without any space');
  }

  $media_id_ary = explode(',', $media_id);

  if (count($media_id_ary) > 5) {
    form_set_error('media_id', 'MediaId(s) should not be more than 5');
  }

  if ($report_range == 4) {
    if ($end_date_form == '') {
      form_set_error('end_date', 'Please enter a valid end date');
    }
    if ($start_date_form == '') {
      form_set_error('start_date', 'Please enter a valid start date');
    }
    if ($start_date_form > $end_date_form && $end_date_form != '') {
      form_set_error('start_date', 'Please enter a valid start date. Start date should not be greater than end date.');
    }
  }

}

/**
 * Implementation of the hook_submit() to save form values
 * @param $form
 * @param $form_state
 *
 */
function _jwplayer_report_form_submit($form, &$form_state) {
  $start_date_form = $form_state['values']['start_date'];
  $end_date_form = $form_state['values']['end_date'];
  $media_id = $form_state['values']['media_id'];
  $start_timestamp = strtotime($start_date_form);
  $end_timestamp = strtotime($end_date_form);
  $start_date = date('Y-m-d', $start_timestamp);
  $end_date = date('Y-m-d', $end_timestamp);
  $option = $form_state['values']['op'];

  $report_range = $form_state['values']['report_range'];

  if ($report_range == 1) {
    $month_val = 6;
  }
  elseif ($report_range == 2) {
    $month_val = 12;
  }
  elseif ($report_range == 3) {
    $month_val = 24;
  }
  elseif (!empty($start_date) && ! empty($end_date)) {
     $month_val = 'custom';
  }

  $new_media_id = str_replace(',', '-', $media_id);
  $goto_url = 'admin/jwplayer/report';
  drupal_goto($goto_url, array("query" => array("media_id" => $new_media_id, "month_val" => $month_val, "option" => $option, "start_date" => $start_date,
  "end_date" => $end_date,  "report_range" => $report_range )));
}

/**
 * Custom function to show and download jwplayer video data.
 *
 */
function list_jwplayerreport() {
  $until = new DateTime();
  $media_id = isset($_GET['media_id']) ? $_GET['media_id'] : '';
  $start_date = isset($_GET['start_date']) ? $_GET['start_date'] : '';
  $end_date = isset($_GET['end_date']) ? $_GET['end_date'] : '';
  $month_val = isset($_GET['month_val']) ? $_GET['month_val'] : '';
  $option = isset($_GET['option']) ? $_GET['option'] : '';
  $report_range = isset($_GET['report_range']) ? $_GET['report_range'] : '';

  if ($month_val == 6) {
    $startdate = date("Y-m-d");
    $enddate = date("Y-m-d", strtotime("-6 months"));
  }
  elseif ($month_val == 12) {
    $startdate = date("Y-m-d");
    $enddate = date("Y-m-d", strtotime("-12 months"));
  }
  elseif ($month_val == 24) {
    $startdate = date("Y-m-d");
    $enddate = date("Y-m-d", strtotime("-24 months"));
  }
  else {
    $startdate = isset($_GET['start_date']) ? $_GET['start_date'] : '';
    $enddate = isset($_GET['end_date']) ? $_GET['end_date'] : '';
  }

  if ($month_val == 'custom') {
    $start = new DateTime($start_date);
    $start = $start->modify('first day of this month');
    $end = new DateTime($end_date);
    $end = $end->modify('first day of next month');
    $interval = new DateInterval('P1M');
    $period = new DatePeriod($start, $interval , $end);

    foreach ($period as $dt) {
      $month[]= $dt->format("Y-m-d") . "<br>\n";
      $strt = $dt->format("Y-m-d");
      $ed = date('Y-m-t', strtotime($strt));
      $date_array[] = array('start_date' => $strt, 'end_date' => $ed);
    }
    $month_val = count($month);
    $startdate = $_GET['start_date'];
    $enddate = $_GET['end_date'];
  }

  if (empty($month_val)) {
    $month_val = 1;
  }

  for ($i = 0; $i < $month_val; $i++) {

    if ($report_range == 4) {
      $start_date = $date_array[$i]['start_date'];
      $end_date = $date_array[$i]['end_date'];
    }
    else {
      $start_date = date("Y-m-d", strtotime( date( 'Y-m-01' ) . " -$i months"));
      $end_date = date('Y-m-t', strtotime( " -$i months"));
    }
    $api_secret_key = variable_get('api_secret_key', '');
    $property_name_key = variable_get('property_name_key', '');

    if ($media_id != '') {
      $media_id_arr = explode("-", $media_id);
      $output = array();
      $j = 0;
    foreach ($media_id_arr as $m_ids) {

      $data = array(
        "start_date" => $start_date,
        "end_date" => $end_date,
        "dimensions" => array("media_id"),
        "metrics" => array(
          array("operation" => "sum", "field" => "embeds"),
          array("operation" => "sum", "field" => "plays"),
          array("operation" => "sum", "field" => "time_watched"),
        ),
        "page_length" => 100,
        "sort" => array(
          array("field" => "plays", "order" => "DESCENDING"),
        ),
      );

      $data["filter"] = array(
        array("field" => "media_id", "operator" => "=", "value" => array($m_ids)),
      );
      $url = "https://api.jwplayer.com/v2/sites/$property_name_key/analytics/queries/";

      $data = json_encode($data);
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_POST, 1);
      curl_setopt($ch, CURLOPT_HTTPHEADER, array("Authorization:$api_secret_key", "Content-Type: application/json"));
      curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
      $output_data = curl_exec($ch);
      curl_close($ch);
      $output[$j] = json_decode($output_data, TRUE);
      $embed = '';
      $views = '';
      $timewatched = '';
      $timewat = '';
      for ($date_push = 0; $date_push <= $i; $date_push++) {
        if (isset($output[$j]['data']['rows'][$date_push])) {
          $embed += $output[$j]['data']['rows'][$date_push][1];
          $views += $output[$j]['data']['rows'][$date_push][2];
          $timewatched += $output[$j]['data']['rows'][$date_push][3];
          $timewat = seconds_to_time($timewatched);
          if (!empty($output[$j]['data']['rows'][$date_push][3])) {
            $output[$j]['data']['rows'][$date_push][3] = seconds_to_time($output[$j]['data']['rows'][$date_push][3]);
          }

          $startdateval = date("m-Y", strtotime($start_date));

            array_unshift($output[$j]['data']['rows'][$date_push], $startdateval);
          }
      }
      $j++;
    }

    }
    else {

      $data = array(
        "start_date" => $start_date,
        "end_date" => $end_date,
        "dimensions" => array("media_id"),
        "metrics" => array(
          array("operation" => "sum", "field" => "embeds"),
          array("operation" => "sum", "field" => "plays"),
          array("operation" => "sum", "field" => "time_watched"),
        ),
        "page_length" => 100,
        "sort" => array(
          array("field" => "plays", "order" => "DESCENDING"),
        ),
      );

      $data = json_encode($data);
      $url = "https://api.jwplayer.com/v2/sites/$property_name_key/analytics/queries/";
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_POST, 1);
      curl_setopt($ch, CURLOPT_HTTPHEADER, array("Authorization:$api_secret_key", "Content-Type: application/json"));
      curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
      $output = curl_exec($ch);
      curl_close($ch);
      $output = json_decode($output, TRUE);
      $arrycount  = count($output['data']['rows']);
      $embed = '';
      $views = '';
      $timewatched = '';
      $timewat = '';
      for ($date_push = 0; $date_push <= $arrycount; $date_push++) {
        if (isset($output['data']['rows'][$date_push])) {
          $embed += $output['data']['rows'][$date_push][1];
          $views += $output['data']['rows'][$date_push][2];
          $timewatched += $output['data']['rows'][$date_push][3];
          $timewat = seconds_to_time($timewatched);

          if (!empty($output['data']['rows'][$date_push][3])) {
            $output['data']['rows'][$date_push][3] = seconds_to_time($output['data']['rows'][$date_push][3]);
          }
          $startdateval = date("m-Y", strtotime($start_date));
          array_unshift($output['data']['rows'][$date_push], $startdateval);
        }
      }
    }

    if ($media_id != '') {
      foreach ($output as $output_data) {
        $reportdata[] = $output_data['data']['rows'];
      }
    }
    else {
      $reportdata[] = $output['data']['rows'];
    }

  }

    if ($option == 'Download') {
      $file = "jwPlayerReport.csv";
      $delimiter = ";";
      $header_media_id = str_replace('-', ',', $media_id);
      header('Content-Type: application/csv');
      header('Content-Disposition: attachment; filename="' . $file . '";');
      $f = fopen('php://output', 'w');
      fputcsv($f, array("Advertiser History"));
      fputcsv($f, array("MediaId(s)", "$header_media_id" ));
      fputcsv($f, array("Start date", "$startdate" ));
      fputcsv($f, array("End date", "$enddate" ));
      fputcsv($f, array("Month", "MediaId", "Embeds", "Views", "Time Watched"));
      foreach ($reportdata as $data) {
        foreach ($data as $line) {
          fputcsv($f, $line);
        }
      }
      fputcsv($f, array("Total", "", "$embed", "$views", "$timewat"));
      exit();
    }
    elseif ($option == 'List') {
      $template_list = '';
      $data =array();
      $header = array(
      array('data' => t('Date')),
      array('data' => t('Media_id')),
      array('data' => t('Embeds')),
      array('data' => t('Views')),
      array('data' => t('Time Watched')),
      );

      foreach ($reportdata as $data) {
        if (isset($data)) {
          foreach ($data as $line) {
            if (!empty($line)) {
              $res_media_id = $line[1];
              $date_month = $line[0];
              $embeds = $line[2];
              $plays = $line[3];
              $time_watched = $line[4];

              $rows[] = array($date_month, $res_media_id, $embeds, $plays, $time_watched);
            }
         }
        }
      }
      if (empty($rows)) {
        $rows[] = array(array('data' => t('No Data Available.'), 'colspan' => 5));
      }

      $header_media_id = str_replace('-', ',', $media_id);
      $template_list .= '<table class="custom-table-details" style="border-collapse: unset;">
                          <tr> <td><br><b>Advertiser History </b></td></tr>
                           <td width="5%"><b>MediaId(s): </b>' . $header_media_id . '</td>
                           <td width="5%"><b>Start date: </b>' . $startdate . '</td>
                           <td width="5%"> <b>End date: </b>' . $enddate . '</td> </tr>
                        </table>';


      $template_list .= theme('table', array('header' => $header, 'rows' => $rows, 'empty' => '', 'attributes' => array('class' => array('table-align-l'))));

      return $template_list;
  }
}

/**
 * Custom function to convert senconds into h:m:s format.
 * @param secs
 * Used second as a parameter
 */
function seconds_to_time($secs) {
   $output = '';
   $time =  gmdate('H:i:s', $secs);
   $output = explode(':', $time);
   $time_str = $output[0] . 'h' . $output[1] . 'm' . $output[2] . 's';

   return $time_str;
}
